@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2011 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2012 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -55,6 +55,15 @@ use base 'RT::SearchBuilder';
 
 sub Table {'Articles'}
 
+sub _Init {
+    my $self = shift;
+    $self->OrderByCols(
+        { FIELD => 'SortOrder', ORDER => 'ASC' },
+        { FIELD => 'Name',      ORDER => 'ASC' },
+    );
+    return $self->SUPER::_Init( @_ );
+}
+
 =head2 Next
 
 Returns the next article that this user can see.
