@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2011 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2012 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -78,6 +78,8 @@ use warnings;
 use RT::Transaction;
 use MIME::Base64;
 use MIME::QuotedPrint;
+use MIME::Body;
+use RT::Util 'mime_recommended_filename';
 
 sub _OverlayAccessible {
   {
@@ -138,7 +140,9 @@ sub Create {
     $MessageId =~ s/^<(.*?)>$/$1/o;
 
     #Get the filename
-    my $Filename = $Attachment->head->recommended_filename;
+
+    my $Filename = mime_recommended_filename($Attachment);
+
     # remove path part. 
     $Filename =~ s!.*/!! if $Filename;
 
@@ -294,14 +298,27 @@ sub Content {
 =head2 OriginalContent
 
 Returns the attachment's content as octets before RT's mangling.
-Currently, this just means restoring text content back to its
+Generally this just means restoring text content back to its
 original encoding.
 
+If the attachment has a C<message/*> Content-Type, its children attachments
+are reconstructed and returned as a string.
+
 =cut
 
 sub OriginalContent {
     my $self = shift;
 
+    # message/* content types represent raw messages.  Since we break them
+    # apart when they come in, we'll reconstruct their child attachments when
+    # you ask for the OriginalContent of the message/ part.
+    if ($self->IsMessageContentType) {
+        # There shouldn't be more than one "subpart" to a message/* attachment
+        my $child = $self->Children->First;
+        return $self->Content unless $child and $child->id;
+        return $child->ContentAsMIME(Children => 1)->as_string;
+    }
+
     return $self->Content unless RT::I18N::IsTextualContentType($self->ContentType);
     my $enc = $self->OriginalEncoding;
 
@@ -419,14 +436,21 @@ sub Quote {
     return (\$body, $max);
 }
 
-=head2 ContentAsMIME
+=head2 ContentAsMIME [Children => 1]
 
 Returns MIME entity built from this attachment.
 
+If the optional parameter C<Children> is set to a true value, the children are
+recursively added to the entity.
+
 =cut
 
 sub ContentAsMIME {
     my $self = shift;
+    my %opts = (
+        Children => 0,
+        @_
+    );
 
     my $entity = MIME::Entity->new();
     foreach my $header ($self->SplitHeaders) {
@@ -439,14 +463,32 @@ sub ContentAsMIME {
         "Content-Type.charset" => $self->OriginalEncoding )
       if $self->OriginalEncoding;
 
-    use MIME::Body;
     $entity->bodyhandle(
         MIME::Body::Scalar->new( $self->OriginalContent )
     );
 
+    if ($opts{'Children'} and not $self->IsMessageContentType) {
+        my $children = $self->Children;
+        while (my $child = $children->Next) {
+            $entity->make_multipart unless $entity->is_multipart;
+            $entity->add_part( $child->ContentAsMIME(%opts) );
+        }
+    }
+
     return $entity;
 }
 
+=head2 IsMessageContentType
+
+Returns a boolean indicating if the Content-Type of this attachment is a
+C<message/> subtype.
+
+=cut
+
+sub IsMessageContentType {
+    my $self = shift;
+    return $self->ContentType =~ m{^\s*message/}i ? 1 : 0;
+}
 
 =head2 Addresses
 
@@ -719,7 +761,7 @@ sub Decrypt {
     my $type = $self->ContentType;
     if ( $type =~ /^x-application-rt\/gpg-encrypted/i ) {
         ($type) = ($type =~ /original-type="(.*)"/i);
-        $type ||= 'application/octeat-stream';
+        $type ||= 'application/octet-stream';
     } else {
         return (1, $self->loc('Is not encrypted'));
     }
