@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2011 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2012 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -50,23 +50,43 @@ package RT::Test::Apache;
 use strict;
 use warnings;
 
-my $apache_module_prefix;
+my %MODULES = (
+    '2.2' => {
+        "mod_perl" => [qw(authz_host env alias perl)],
+        "fastcgi"  => [qw(authz_host env alias mime fastcgi)],
+    },
+);
+
+my $apache_module_prefix = $ENV{RT_TEST_APACHE_MODULES};
 my $apxs =
      $ENV{RT_TEST_APXS}
   || RT::Test->find_executable('apxs')
   || RT::Test->find_executable('apxs2');
 
-if ($apxs) {
+if ($apxs and not $apache_module_prefix) {
     $apache_module_prefix = `$apxs -q LIBEXECDIR`;
     chomp $apache_module_prefix;
 }
 
 $apache_module_prefix ||= 'modules';
 
+sub basic_auth {
+    my $self = shift;
+    my $passwd = File::Spec->rel2abs( File::Spec->catfile(
+        't', 'data', 'configs', 'passwords' ) );
+
+    return <<"EOT";
+    AuthType Basic
+    AuthName "restricted area"
+    AuthUserFile $passwd
+    Require user root
+EOT
+}
+
 sub start_server {
-    my ($self, $variant, $port, $tmp) = @_;
-    my %tmp = %$tmp;
-    my %info = $self->apache_server_info( variant => $variant );
+    my ($self, %config) = @_;
+    my %tmp = %{$config{tmp}};
+    my %info = $self->apache_server_info( %config );
 
     RT::Test::diag(do {
         open( my $fh, '<', $tmp{'config'}{'RT'} ) or die $!;
@@ -76,10 +96,10 @@ sub start_server {
 
     my $tmpl = File::Spec->rel2abs( File::Spec->catfile(
         't', 'data', 'configs',
-        'apache'. $info{'version'} .'+'. $variant .'.conf'
+        'apache'. $info{'version'} .'+'. $config{variant} .'.conf'
     ) );
     my %opt = (
-        listen         => $port,
+        listen         => $config{port},
         server_root    => $info{'HTTPD_ROOT'} || $ENV{'HTTPD_ROOT'}
             || Test::More::BAIL_OUT("Couldn't figure out server root"),
         document_root  => $RT::MasonComponentRoot,
@@ -87,16 +107,15 @@ sub start_server {
         rt_bin_path    => $RT::BinPath,
         rt_sbin_path   => $RT::SbinPath,
         rt_site_config => $ENV{'RT_SITE_CONFIG'},
+        load_modules   => $info{load_modules},
+        basic_auth     => $config{basic_auth} ? $self->basic_auth : "",
     );
     foreach (qw(log pid lock)) {
         $opt{$_ .'_file'} = File::Spec->catfile(
             "$tmp{'directory'}", "apache.$_"
         );
     }
-    {
-        my $method = 'apache_'.$variant.'_server_options';
-        $self->$method( \%info, \%opt );
-    }
+
     $tmp{'config'}{'apache'} = File::Spec->catfile(
         "$tmp{'directory'}", "apache.conf"
     );
@@ -108,16 +127,24 @@ sub start_server {
 
     $self->fork_exec($info{'executable'}, '-f', $tmp{'config'}{'apache'});
     my $pid = do {
-        my $tries = 10;
-        while ( !-e $opt{'pid_file'} ) {
+        my $tries = 15;
+        while ( !-s $opt{'pid_file'} ) {
             $tries--;
             last unless $tries;
             sleep 1;
         }
-        Test::More::BAIL_OUT("Couldn't start apache server, no pid file")
-            unless -e $opt{'pid_file'};
-        open( my $pid_fh, '<', $opt{'pid_file'} )
-            or Test::More::BAIL_OUT("Couldn't open pid file: $!");
+        my $pid_fh;
+        unless (-e $opt{'pid_file'} and open($pid_fh, '<', $opt{'pid_file'})) {
+            Test::More::BAIL_OUT("Couldn't start apache server, no pid file (unknown error)")
+                  unless -e $opt{log_file};
+
+            open my $log, "<", $opt{log_file};
+            my $error = do {local $/; <$log>};
+            close $log;
+            $RT::Logger->error($error) if $error;
+            Test::More::BAIL_OUT("Couldn't start apache server!");
+        }
+
         my $pid = <$pid_fh>;
         chomp $pid;
         $pid;
@@ -135,6 +162,11 @@ sub apache_server_info {
         || $self->find_apache_server
         || Test::More::BAIL_OUT("Couldn't find apache server, use RT_TEST_APACHE");
 
+    Test::More::BAIL_OUT(
+        "Couldn't find apache modules directory (set APXS= or RT_TEST_APACHE_MODULES=)"
+    ) unless -d $apache_module_prefix;
+
+
     RT::Test::diag("Using '$bin' apache executable for testing");
 
     my $info = `$bin -V`;
@@ -152,49 +184,30 @@ sub apache_server_info {
         split /\r*\n/, `$bin -l`
     ];
 
+    Test::More::BAIL_OUT(
+        "Unsupported apache version $res{version}"
+    ) unless exists $MODULES{$res{version}};
+
+    Test::More::BAIL_OUT(
+        "Unsupported apache variant $res{variant}"
+    ) unless exists $MODULES{$res{version}}{$res{variant}};
+
+    my @mlist = @{$MODULES{$res{version}}{$res{variant}}};
+    push @mlist, "authn_file", "auth_basic", "authz_user" if $res{basic_auth};
+
+    $res{'load_modules'} = '';
+    foreach my $mod ( @mlist ) {
+        next if grep $_ =~ /^(mod_|)$mod\.c$/, @{ $res{'modules'} };
+
+        my $so_file = $apache_module_prefix."/mod_".$mod.".so";
+        Test::More::BAIL_OUT( "Couldn't load $mod module (expected in $so_file)" )
+              unless -f $so_file;
+        $res{'load_modules'} .=
+            "LoadModule ${mod}_module $so_file\n";
+    }
     return %res;
 }
 
-sub apache_mod_perl_server_options {
-    my $self = shift;
-    my %info = %{ shift() };
-    my $current = shift;
-
-    my %required_modules = (
-        '2.2' => [qw(authz_host env alias perl)],
-    );
-    my @mlist = @{ $required_modules{ $info{'version'} } };
-
-    $current->{'load_modules'} = '';
-    foreach my $mod ( @mlist ) {
-        next if grep $_ =~ /^(mod_|)$mod\.c$/, @{ $info{'modules'} };
-
-        $current->{'load_modules'} .=
-            "LoadModule ${mod}_module $apache_module_prefix/mod_${mod}.so\n";
-    }
-    return;
-}
-
-sub apache_fastcgi_server_options {
-    my $self = shift;
-    my %info = %{ shift() };
-    my $current = shift;
-
-    my %required_modules = (
-        '2.2' => [qw(authz_host env alias mime fastcgi)],
-    );
-    my @mlist = @{ $required_modules{ $info{'version'} } };
-
-    $current->{'load_modules'} = '';
-    foreach my $mod ( @mlist ) {
-        next if grep $_ =~ /^(mod_|)$mod\.c$/, @{ $info{'modules'} };
-
-        $current->{'load_modules'} .=
-            "LoadModule ${mod}_module $apache_module_prefix/mod_${mod}.so\n";
-    }
-    return;
-}
-
 sub find_apache_server {
     my $self = shift;
     return $_ foreach grep defined,
@@ -203,9 +216,19 @@ sub find_apache_server {
     return undef;
 }
 
+sub apache_mpm_type {
+    my $self = shift;
+    my $apache = $self->find_apache_server;
+    my $out = `$apache -l`;
+    if ( $out =~ /^\s*(worker|prefork|event|itk)\.c\s*$/m ) {
+        return $1;
+    }
+}
+
 sub fork_exec {
     my $self = shift;
 
+    RT::Test::__disconnect_rt();
     my $pid = fork;
     unless ( defined $pid ) {
         die "cannot fork: $!";
@@ -213,6 +236,7 @@ sub fork_exec {
         exec @_;
         die "can't exec `". join(' ', @_) ."` program: $!";
     } else {
+        RT::Test::__reconnect_rt();
         return $pid;
     }
 }
